/*
 *  PCUServeur for the PCU project.
 *  Copyright (C) 2010  PLU Julien
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  For questions: julien.plu@redaction-developpez.com
 */
package pcuserveur.lib;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map.Entry;

/**
 * @author PLU Julien
 * @version 1.0 
 * 
 * <b>
 *     Un canal est un groupe de discussion plusieurs-à-plusieurs.
 *     Identifié par un nom, unique et sans espaces, il possède également
 *     un sujet qui le décrit brièvement, et la liste des gestionnaires de
 *     connexion formant le groupe. Sa signature est le triplet (
 *     <code>nom</code>, "<code>sujet</code>", <code>nombreMembres</code>).
 * </b>
 * <p>
 *     Un Canal est caractérisé par les informations suivantes:
 *     <ul>
 *	       <li>Une HashMap ayant un String pour clef et un
 *	       GestionnaireConnexion pour valeur.</li>
 *         <li>Un String pour le nom du canal.</li>
 *     	   <li>Un String pour le sujet du canal.</li>
 *     </ul>
 * </p>
 *          
 * @see GestionnaireConnexion
 * @see Evenement
 * @see Serveur
 * @see Vide
 * @see DejaConnecteException
 * @see NonConnecteException
 */
public class Canal implements Vide {
	/**
	 * <p>
	 * Liste des connexions contenues dans le canal.
	 * </p>
	 * 
	 * @see Canal#getSignature ()
	 * @see Canal#ajouterGC (GestionnaireConnexion)
	 * @see Canal#enleverGC (GestionnaireConnexion)
	 * @see Canal#enleverGC (String)
	 * @see Canal#listerMembre ()
	 * @see Canal#notifier (Evenement, String, String)
	 * @see Canal#vider ()
	 */
	private final HashMap<String, GestionnaireConnexion> listeGestionnaires = new HashMap<String, GestionnaireConnexion> ();
	/**
	 * <p>
	 * Nom du canal.
	 * </p>
	 * 
	 * @see Canal#getNom ()
	 * @see Canal#Canal (String, String)
	 * @see Canal#estVide ()
	 * @see Canal#initialiser (String, String)
	 * @see Canal#notifier (Evenement, String, String)
	 * @see Canal#getSignature ()
	 * @see Canal#vider ()
	 */
	private String nom;
	/**
	 * <p>
	 * Sujet du canal.
	 * </p>
	 * 
	 * @see Canal#getSujet ()
	 * @see Canal#Canal (String, String)
	 * @see Canal#changerSujet (String)
	 * @see Canal#estVide ()
	 * @see Canal#initialiser (String, String)
	 * @see Canal#getSignature ()
	 * @see Canal#vider ()
	 */
	private String sujet;

	/**
	 * <p>
	 * Constructeur par défaut.
	 * </p>
	 */
	public Canal () {
		// Constructeur par défaut
	}

	/**
	 * <p>
	 * Constructeur paramétré.
	 * </p>
	 * 
	 * @param nom1
	 *            Le nom.
	 * @param sujet1
	 *            Le sujet.
	 */
	public Canal (final String nom1, final String sujet1) {
		this.nom = nom1;
		this.sujet = sujet1;
	}

	/**
	 * <p>
	 * Ajoute un gestionnaire de connexion au canal. Après avoir ajouté
	 * <code>gc</code>, les autres membres du canal sont notifiés de son
	 * arrivée.
	 * </p>
	 * 
	 * @param gc
	 *            Le gestionnaire de connexion à ajouter.
	 *            
	 * @throws DejaConnecteException
	 *             <code>gc<code> est déjà dans le canal.
	 * @throws IOException
	 *             Un envoie lors de la notification a échoué.
	 */
	public void ajouterGC (final GestionnaireConnexion gc) throws DejaConnecteException, IOException {
		final String login = gc.getClient ().getLogin ();

		if (!this.listeGestionnaires.containsKey (login)) {
			this.listeGestionnaires.put (login, gc);
			this.notifier (Evenement.REJOINT, login, ""); //$NON-NLS-1$
		}
		else {
			throw new DejaConnecteException (login + " est deja dans " + this.nom); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Change le sujet du canal. Après avoir changé le sujet, les autres membres
	 * du canal sont notifiés du changement.
	 * </p>
	 * 
	 * @param sujet1
	 *            Le nouveau sujet.
	 *            
	 * @throws IOException
	 *             Un envoie lors de la notification a échoué.
	 */
	public void changerSujet (final String sujet1) throws IOException {
		this.sujet = sujet1;

		this.notifier (Evenement.SUJETCHANGE, "", sujet1); //$NON-NLS-1$
	}

	/**
	 * <p>
	 * Enlève un gestionnaire de connexion du canal. Après avoir enlevé
	 * <code>gc</code>, les autres membres du canal sont notifiés de son départ.
	 * </p>
	 * 
	 * @param gc
	 *            Le gestionnaire de connexion à retirer.
	 *            
	 * @throws NonConnecteException
	 *             <code>gc</code> n'est pas dans le canal.
	 * @throws IOException
	 *             Un envoie lors de la notification a échoué.
	 */
	public synchronized void enleverGC (final GestionnaireConnexion gc) throws NonConnecteException, IOException {
		final String login = gc.getClient ().getLogin ();

		if (this.listeGestionnaires.containsKey (login)) {
			this.listeGestionnaires.remove (login);
			this.notifier (Evenement.PARTI, login, ""); //$NON-NLS-1$
		}
		else {
			throw new NonConnecteException (login + " n'est pas dans " + this.nom); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Enlève un gestionnaire de connexion du canal. Après avoir enlevé
	 * <code>login</code>, les autres membres du canal sont notifiés de son
	 * départ.
	 * </p>
	 * 
	 * @param login
	 *            Login du client à enlever.
	 *            
	 * @throws NonConnecteException
	 *             <code>login</code> n'est pas dans le canal.
	 */
	public synchronized void enleverGC (final String login) throws NonConnecteException {
		if (this.listeGestionnaires.containsKey (login)) {
			this.listeGestionnaires.remove (login);
		}
		else {
			throw new NonConnecteException (login + " n'est pas dans " + this.nom); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Savoir si le canal est vide.
	 * </p>
	 * 
	 * @return <code>true</code> si le canal est vide, <code>false</code> sinon.
	 */
	public boolean estVide () {
		return (this.nom == null) && (this.sujet == null);
	}

	/**
	 * <p>
	 * Obtenir le nom du canal, si il existe.
	 * </p>
	 * 
	 * @return Le nom, ou <code>null</code> si le canal est vide.
	 */
	public String getNom () {
		return this.nom;
	}

	/**
	 * <p>
	 * Obtenir la signature du canal.
	 * </p>
	 * 
	 * @return La signature du canal, ou <code>null</code> si il est vide.
	 */
	public String getSignature () {
		return this.estVide () ? null : this.nom + " \"" + this.sujet + "\" " + this.listeGestionnaires.size (); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * <p>
	 * Obtenir le sujet du canal, si il existe.
	 * </p>
	 * 
	 * @return Le sujet, ou <code>null</code> si le canal est vide.
	 */
	public String getSujet () {
		return this.sujet;
	}

	/**
	 * <p>
	 * Initialise le canal, à la façon du constructeur paramétré.
	 * </p>
	 * 
	 * @param nom1
	 *            Le nom.
	 * @param sujet1
	 *            Le sujet.
	 */
	public void initialiser (final String nom1, final String sujet1) {
		this.nom = nom1;
		this.sujet = sujet1;
	}

	/**
	 * <p>
	 * Initialise le canal, à la façon du constructeur paramétré.
	 * </p>
	 * 
	 * @return la liste des membres du canal
	 */
	public String listerMembre () {
		final StringBuffer res = new StringBuffer ();
		int i = 0;

		for (final Entry<String, GestionnaireConnexion> entry:this.listeGestionnaires.entrySet ()) {
			final GestionnaireConnexion valeur = entry.getValue ();

			res.append (valeur.getClient ().getLogin ());
			if (i != this.listeGestionnaires.size () - 1) {
				res.append (" "); //$NON-NLS-1$
			}
			i++;
		}

		return res.toString ();
	}

	/**
	 * <p>
	 * Notifie tous les membres du canal, d'un évènement l'affectant.
	 * </p>
	 * 
	 * @param evenement
	 *            L'événement à notifier.
	 * @param login
	 *            Le login. Si il n'est pas pertinant, la chaîne est vide.
	 * @param texte
	 *            Le texte. Si il n'est pas pertinant, la chaîne est vide.
	 *            
	 * @throws IOException
	 *             Un envoie lors de la notification a échoué.
	 */
	public synchronized void notifier (final Evenement evenement, final String login, final String texte) throws IOException {
		for (final Entry<String, GestionnaireConnexion> entry:this.listeGestionnaires.entrySet ()) {
			final GestionnaireConnexion valeur = entry.getValue ();

			if (valeur != null) {
				valeur.notifier (this.nom, evenement, login, texte);
			}
		}
	}

	/**
	 * <p>
	 * Vide le canal. Positionne le nom et le sujet à <code>null</code>, et
	 * supprime tous les éléments de la liste des gestionnaires de connexion. A
	 * utiliser avec beaucoup de précaution.
	 * </p>
	 * 
	 * @see Vide
	 */
	public void vider () {
		this.nom = null;
		this.sujet = null;
		this.listeGestionnaires.clear ();
	}
}
