/*
 *  PCUServeur for the PCU project.
 *  Copyright (C) 2010  PLU Julien
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  For questions: julien.plu@redaction-developpez.com
 */
package pcuserveur.lib;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.NoSuchAlgorithmException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

/**
 * @author PLU Julien
 * @version 1.0 
 * 
 * <b>
 *     Cette classe implémente le protocole PCU et prend en charge
 *     la connexion entre le serveur et le client. 
 * </b>
 * <p>
 *     Un GestionnaireConnexion est caractérisé par les informations
 *     suivantes:
 *     <ul>
 *         <li>Code du caractère CR.</li>
 *         <li>Code du caractère CRLF.</li>
 *         <li>Code de la séquence LF.</li>
 *         <li>Un Utilisateur associé au gestionnaire de connexion.</li>
 *         <li>Un flux d'entrée du gestionnaire de connexion.</li>
 *         <li>Un Set contenant les états dans lequel ce trouve l'utilisateur.</li>
 *         <li>Un boolean représentant le fonctionnement du thread.</li>
 *         <li>Une HashMap ayant un String pour clef et un Canal pour valeur.</li>
 *         <li>Un int pour le numéro du gestionnaire de connexion.</li>
 *         <li>Un serveur.</li>
 *         <li>Une Socket.</li>
 *         <li>Flux de sortie du gestionnaire de connexion.</li>
 *         <li>Une ArrayListe pour la liste des login des client pour lesquels
 *          on a accepté les connexion directe accepte.</li>
 *         <li>Une ArrayListe pour la liste des login des client pour lesquels
 *          on a refusé les connexion directe accepte.</li>
 *     </ul>
 * </p>
 * 
 * @see Runnable
 * @see Vide
 * @see Utilisateur
 * @see Etat
 * @see Canal
 * @see Serveur
 * @see MonLog
 * @see DejaConnecteException
 * @see CanalExistantException
 * @see CommandeIncorrecteException
 * @see LoginIntrouvableException
 * @see NonConnecteException
 * @see NbCanauxMaxException
 */
public class GestionnaireConnexion implements Vide , Runnable {
	/**
	 * <p>
	 * Caractère représentant le caractère "Cariage Return".
	 * </p>
	 * 
	 * @see GestionnaireConnexion#lister (String)
	 * @see GestionnaireConnexion#membre (String)
	 * @see GestionnaireConnexion#srv_lister ()
	 * @see Serveur#informationMembre (String)
	 * @see Serveur#listerCanaux (String)
	 * @see Serveur#listerServeur ()
	 */
	public static final char CR = '\r';
	/**
	 * <p>
	 * Caractère représentant la chaîne "Cariage Return Line Feed".
	 * </p>
	 * 
	 * @see GestionnaireConnexion#envoyer (String)
	 */
	private static final String CRLF = GestionnaireConnexion.CR + "" + GestionnaireConnexion.LF; //$NON-NLS-1$
	/**
	 * <p>
	 * Caractère représentant le caractère "Line Feed".
	 * </p>
	 */
	private static final char LF = '\n';
	/**
	 * <p>
	 * Utilisateur correspondant au gestionnaire de connexion actuel.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#cnl_changerSujet (String, String)
	 * @see GestionnaireConnexion#cnl_creer (String, String)
	 * @see GestionnaireConnexion#cnl_supprimer (String)
	 * @see GestionnaireConnexion#getClient ()
	 * @see GestionnaireConnexion#message (String, String)
	 * @see GestionnaireConnexion#quitter (String)
	 * @see GestionnaireConnexion#run ()
	 * @see GestionnaireConnexion#srv_lister ()
	 * @see GestionnaireConnexion#srv_relancer(String)
	 * @see GestionnaireConnexion#stopper (String)
	 * @see GestionnaireConnexion#utl_ejecter (String, String)
	 * @see GestionnaireConnexion#vider ()
	 */
	private final Utilisateur client = new Utilisateur ();
	/**
	 * <p>
	 * Flux d'entrée du gestionnaire de connexion actuel.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#run ()
	 * @see GestionnaireConnexion#vider ()
	 */
	private BufferedReader entree;
	/**
	 * <p>
	 * Etat(s) de l'utilisateur actuel.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#cnl_changerSujet (String, String)
	 * @see GestionnaireConnexion#cnl_creer (String, String)
	 * @see GestionnaireConnexion#cnl_supprimer (String)
	 * @see GestionnaireConnexion#identifier (String, String)
	 * @see GestionnaireConnexion#infoMembre (String)
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#lister (String)
	 * @see GestionnaireConnexion#membre (String)
	 * @see GestionnaireConnexion#message (String, String)
	 * @see GestionnaireConnexion#quitter (String)
	 * @see GestionnaireConnexion#quitterCanal (String)
	 * @see GestionnaireConnexion#rejoindre (String)
	 * @see GestionnaireConnexion#srv_lister ()
	 * @see GestionnaireConnexion#srv_relancer (String)
	 * @see GestionnaireConnexion#srv_stopper (String)
	 * @see GestionnaireConnexion#utl_ejecter (String, String)
	 * @see GestionnaireConnexion#vider ()
	 */
	private final Set<Etat> etats = new HashSet<Etat> ();
	/**
	 * <p>
	 * Gestion du thread.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#run ()
	 * @see GestionnaireConnexion#setLance (boolean)
	 * @see GestionnaireConnexion#vider ()
	 */
	private boolean lance = true;
	/**
	 * <p>
	 * Liste des canaux ou l'utilisateur actuel se situe.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#enleverCanal (String)
	 * @see GestionnaireConnexion#infoCanaux ()
	 * @see GestionnaireConnexion#membre (String)
	 * @see GestionnaireConnexion#message (String, String)
	 * @see GestionnaireConnexion#quitter (String)
	 * @see GestionnaireConnexion#quitterCanal (String)
	 * @see GestionnaireConnexion#rejoindre (String)
	 * @see GestionnaireConnexion#vider ()
	 */
	private final HashMap<String, Canal> listeCanaux = new HashMap<String, Canal> ();
	/**
	 * <p>
	 * Liste des login avec lesquels on a accepté les connections directes
	 * </p>
	 * 
	 * @see GestionnaireConnexion#connexionDirecte (String, String)
	 * @see GestionnaireConnexion#connexionDirecteAccepte (String, String, String)
	 * @see GestionnaireConnexion#connexionDirecteRefuse (String)
	 */
	private final ArrayList<String> listeNomAccepte = new ArrayList<String> ();
	/**
	 * <p>
	 * Liste des login avec lesquels on a refusé les connections directes
	 * </p>
	 * 
	 * @see GestionnaireConnexion#connexionDirecte (String, String)
	 * @see GestionnaireConnexion#connexionDirecteAccepte (String, String, String)
	 * @see GestionnaireConnexion#connexionDirecteRefuse (String)
	 */
	private final ArrayList<String> listeNomRefuse = new ArrayList<String> ();
	/**
	 * <p>
	 * Numéro du gestionnaire de connexion actuel dans la liste du serveur.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#quitter (String)
	 * @see GestionnaireConnexion#vider ()
	 */
	private int numeroGC;
	/**
	 * <p>
	 * Serveur auquel le gestionnaire de connexion actuel est connecté.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#cnl_changerSujet (String, String)
	 * @see GestionnaireConnexion#cnl_creer (String, String)
	 * @see GestionnaireConnexion#cnl_supprimer (String)
	 * @see GestionnaireConnexion#identifier (String, String)
	 * @see GestionnaireConnexion#infoMembre (String)
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#lister (String)
	 * @see GestionnaireConnexion#membre (String)
	 * @see GestionnaireConnexion#message (String, String)
	 * @see GestionnaireConnexion#quitter (String)
	 * @see GestionnaireConnexion#quitterCanal (String)
	 * @see GestionnaireConnexion#rejoindre (String)
	 * @see GestionnaireConnexion#srv_lister ()
	 * @see GestionnaireConnexion#srv_relancer (String)
	 * @see GestionnaireConnexion#srv_stopper (String)
	 * @see GestionnaireConnexion#utl_ejecter (String, String)
	 * @see GestionnaireConnexion#vider ()
	 */
	private Serveur serveur;

	/**
	 * <p>
	 * Socket sur lequel le gestionnaire de connexion recevra et émettra les
	 * données.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#estVide ()
	 * @see GestionnaireConnexion#getIp ()
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#vider ()
	 */
	private Socket socket;

	/**
	 * <p>
	 * Flux de sortie du gestionnaire de connexion actuel.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#envoyer (String)
	 * @see GestionnaireConnexion#initialisationGestionnaire (Socket, Serveur,
	 *      int)
	 * @see GestionnaireConnexion#run ()
	 * @see GestionnaireConnexion#vider ()
	 */
	private BufferedWriter sortie;

	/**
	 * <p>
	 * Constructeur par défaut.
	 * </p>
	 */
	public GestionnaireConnexion () {
		// Constructeur par defaut.
	}

	/**
	 * <p>
	 * Analyse des messages envoyés par le client.
	 * </p>
	 * 
	 * @param message
	 *            Message envoyé par le client à analyser.
	 *            
	 * @return La Liste contenant l'analyse final du message envoyé par
	 *         le client.
	 *         
	 * @throws CommandeIncorrecteException
	 *             La commande rentrée est incorrect.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized List<String> analyseMessage (final String message) throws CommandeIncorrecteException, IOException {
		final List<String> res = Collections.synchronizedList (new ArrayList<String> ());
		final StringBuffer tmp = new StringBuffer ();
		boolean b = false;
		int i = 0, nbg = 0;

		if (message == null) {
			throw new IOException ();
		}
		// On compte le nombre de guillemets
		while (i < message.length ()) {
			if (message.charAt (i) == '"') {
				nbg++;
			}
			i++;
		}
		// Si celui ci est impair on leve une exception
		if (nbg % 2 != 0) {
			throw new CommandeIncorrecteException ();
		}

		i = 0;

		// On parcours toute la chaine
		while (i < message.length ()) {
			// Si on tombe sur un guillemet on bascule le booleen b et on avance
			if (message.charAt (i) == '"') {
				b = !b;
				i++;
			}
			// si on est a l'interieur de guillemets on memorise tout jusqu'au
			// prochain guillemet
			if (b) {
				while ((i < message.length ()) && (message.charAt (i) != '"')) {
					tmp.append (message.charAt (i));
					i++;
				}
				res.add (tmp.toString ());
				tmp.delete (0, tmp.length ());
			}
			else {
				// tant qu'on trouve des espaces on avance
				while ((i < message.length ()) && (message.charAt (i) == ' ')) {
					i++;
				}
				// tant qu'il n'y a pas d'espace ni de guillemet on avance et on
				// memorise
				while ((i < message.length ()) && (message.charAt (i) != ' ') && (message.charAt (i) != '"')) {
					tmp.append (message.charAt (i));
					i++;
				}
				// Si on a memorise quelquechose on le met dans le vecteur
				if (!tmp.toString ().equals ("")) {//$NON-NLS-1$
					res.add (tmp.toString ());
				}
				tmp.delete (0, tmp.length ());
			}
		}
		if (res.size () == 0) {
			throw new CommandeIncorrecteException ();
		}

		return res;
	}

	/**
	 * <p>
	 * Changer le sujet d'un canal.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal qui verra son sujet changé.
	 * @param sujet
	 *            Nouveau sujet du canal.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void cnl_changerSujet (final String nom, final String sujet) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				try {
					this.serveur.changerSujetCanal (nom, sujet);

					reponse = "+OK_CNL_CHANGERSUJET " + nom + " \"" + sujet + "\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				catch (final CanalInexistantException e) {
					reponse = "-ERR_CANALINTROUVABLE"; //$NON-NLS-1$
				}
				reponse += " " + nom; //$NON-NLS-1$
			}
			else {
				reponse = "-ERR_NONADMINISTRATEUR"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Creer un nouveau canal.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du nouveau canal.
	 * @param sujet
	 *            Sujet du nouveau canal.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void cnl_creer (final String nom, final String sujet) throws IOException {
		String reponse = ""; //$NON-NLS-1$
		boolean nbc = false;

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				try {
					this.serveur.creerCanal (nom, sujet);

					reponse = "+OK_CNL_CREER " + nom + " \"" + sujet + "\""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				}
				catch (final CanalExistantException e) {
					reponse = "-ERR_CANALEXISTANT"; //$NON-NLS-1$
				}
				catch (final NbCanauxMaxException e1) {
					nbc = true;
				}
			}
			else {
				reponse = "-ERR_NONADMINISTRATEUR"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		if (nbc) {
			reponse = "-ERR_NBCANAUXMAX"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Suppresion d'un canal.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal à supprimer.
	 * @throws IOException
	 *            Un envoie à un client a échoué.
	 */
	private synchronized void cnl_supprimer (final String nom) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				try {
					this.serveur.supprimerCanal (nom);

					reponse = "+OK_CNL_SUPPRIMER"; //$NON-NLS-1$
				}
				catch (final CanalInexistantException e) {
					reponse = "-ERR_CANALINTROUVABLE"; //$NON-NLS-1$
				}
				reponse += " " + nom; //$NON-NLS-1$
			}
			else {
				reponse = "-ERR_NONADMINISTRATEUR"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Demande de connexion directe avec quelqu'un.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @param ip
	 *            Ip de la personne qui demande la connexion directe.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void connexionDirecte (final String login, final String ip) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			try {
				if (!this.listeNomAccepte.contains (login)) {
					if (this.listeNomRefuse.contains (login)) {
						this.listeNomRefuse.remove (login);
					}
					GestionnaireConnexion gc = new GestionnaireConnexion ();
					
					gc = this.serveur.connexion_directe (login);

					gc.envoyer ("CONNEXIONDIRECTE " + this.client.getLogin () + " " + ip); //$NON-NLS-1$//$NON-NLS-2$

					this.listeNomAccepte.add (login);

					gc = null;
				}
			}
			catch (final LoginIntrouvableException e) {
				reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Acceptation d'une connexion directe avec quelqu'un.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @param ip
	 *            Ip de la personne qui demande la connexion directe.
	 * @param port_connexion
	 *            Port sur lequel la connexion sera établie.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void connexionDirecteAccepte (final String login, final String ip, final String port_connexion) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			try {
				if (!this.listeNomAccepte.contains (login)) {
					if (this.listeNomRefuse.contains (login)) {
						this.listeNomRefuse.remove (login);
					}
					this.listeNomAccepte.add (login);
					this.etats.add (Etat.connexion_directe);

					GestionnaireConnexion gc = new GestionnaireConnexion ();
					
					gc = this.serveur.connexion_directe_accepte (login);

					gc.envoyer ("+OK_CONNEXIONDIRECTE " + ip + " " + port_connexion); //$NON-NLS-1$//$NON-NLS-2$

					gc = null;
				}
			}
			catch (final LoginIntrouvableException e) {
				reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Refus d'une connexion directe avec quelqu'un.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void connexionDirecteRefuse (final String login) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			try {
				if (!this.listeNomRefuse.contains (login)) {
					this.listeNomRefuse.add (login);
					this.etats.add (Etat.connexion_directe);
					
					GestionnaireConnexion gc = new GestionnaireConnexion ();

					gc = this.serveur.connexion_directe_refuse (login);

					gc.envoyer ("+KO_CONNEXIONDIRECTE " + login); //$NON-NLS-1$
					gc.enleverNom (login);

					gc = null;
				}
			}
			catch (final LoginIntrouvableException e) {
				reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Enlever un canal de la liste des canaux ou se situe l'utilisateur.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal à enlever de la liste.
	 */
	public void enleverCanal (final String nom_canal) {
		this.listeCanaux.remove (nom_canal);
	}

	/**
	 * <p>
	 * Enlever un nom dans la liste des clients acceptés.
	 * </p>
	 * 
	 * @param login
	 * 			Login à retirer.
	 */
	public void enleverNom (final String login) {
		this.listeNomAccepte.remove (login);
	}

	/**
	 * <p>
	 * Envoyer un message.
	 * </p>
	 * 
	 * @param message
	 *            Message à envoyer.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void envoyer (final String message) throws IOException {
		this.sortie.write (message + GestionnaireConnexion.CRLF);
		this.sortie.flush ();
	}

	/**
	 * <p>
	 * Savoir si un GestionnaireConnexion est vide, c'est à dire si il est
	 * inutilisé.
	 * </p>
	 * 
	 * @return true si le GestionnaireConnexion est vide, false sinon.
	 */
	public synchronized boolean estVide () {
		return this.socket == null;
	}

	/**
	 * <p>
	 * Renvoie le client courant.
	 * </p>
	 * 
	 * @return Le client.
	 */
	public synchronized Utilisateur getClient () {
		return this.client;
	}

	/**
	 * <p>
	 * Renvoie l'adresse IP du client.
	 * </p>
	 * 
	 * @return L'adresse IP.
	 */
	public String getIp () {
		return this.socket.getInetAddress ().toString ().substring (1);
	}

	/**
	 * <p>
	 * Identification du client dans la base de donnée.
	 * </p>
	 * 
	 * @param login
	 *            Login du client.
	 * @param mdp
	 *            Mot de passe du client.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void identifier (final String login, final String mdp) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.non_authentifie)) {
			try {
				this.serveur.verifierLoginMdp (login, mdp, this.client);
				this.etats.remove (Etat.non_authentifie);
				this.etats.add (Etat.authentifie);
				reponse = "+OK_IDENTIFIER " + login + " " + (this.client.estAdministrateur () ? "admin" : "nonadmin"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			}
			catch (final SQLException e) {
				Serveur.logServeur.logMessage ("Erreur avec la base de donnée"); //$NON-NLS-1$
				Serveur.logServeur.logException ("GestionnaireConnexion", "identifier", e); //$NON-NLS-1$ //$NON-NLS-2$
				reponse = "-ERR_INTERNE"; //$NON-NLS-1$
			}
			catch (final IdentifierException e) {
				reponse = "-ERR_ECHECIDENT"; //$NON-NLS-1$
				Serveur.logCoDeco.logMessage (login + " n'a pas pu se connecté"); //$NON-NLS-1$
			}
			catch (final NoSuchAlgorithmException e) {
				Serveur.logServeur.logMessage ("Algorithme non trouvé"); //$NON-NLS-1$
				Serveur.logServeur.logException ("GestionnaireConnexion", "identifier", e); //$NON-NLS-1$ //$NON-NLS-2$
				reponse = "-ERR_INTERNE"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Renvoie le nom des canaux auxquels le client est connecté.
	 * </p>
	 * 
	 * @return Chaine des nom de canaux.
	 */
	public String infoCanaux () {
		final StringBuffer res = new StringBuffer ();
		final int i = 0;

		for (final Entry<String, Canal> entry:this.listeCanaux.entrySet ()) {
			final Canal valeur = entry.getValue ();

			res.append (valeur.getNom ());
			if (i != this.listeCanaux.size () - 1) {
				res.append (" "); //$NON-NLS-1$
			}
		}

		return res.toString ();
	}

	/**
	 * <p>
	 * Connaitre les informations de tout les utilisateurs connectés au serveur.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur dont on veut connaître les
	 *            informations.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void infoMembre (final String login) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			try {
				reponse = "+OK_INFOMEMBRE " + this.serveur.informationMembre (login); //$NON-NLS-1$
			}
			catch (final LoginIntrouvableException e) {
				reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Initialise le gestionnaire de connexion.
	 * </p>
	 * 
	 * @param socket1
	 *            Socket ou le gestionnaire de connexion sera connecté.
	 * @param serveur1
	 *            Serveur sur lequel le gestionnaire de connexion sera connecté.
	 * @param numeroGC1
	 *            Numéro du gestionnaire de connexion dans le serveur.
	 */
	public synchronized void initialisationGestionnaire (final Socket socket1, final Serveur serveur1, final int numeroGC1) {
		this.socket = socket1;
		this.serveur = serveur1;
		this.numeroGC = numeroGC1;
		this.lance = true;

		try {
			this.etats.add (Etat.non_authentifie);
			this.sortie = new BufferedWriter (new OutputStreamWriter (this.socket.getOutputStream (), "UTF-8")); //$NON-NLS-1$
			this.entree = new BufferedReader (new InputStreamReader (this.socket.getInputStream (), "UTF-8")); //$NON-NLS-1$
		}
		catch (final UnknownHostException e) {
			Serveur.logServeur.logMessage ("Hote inconnu"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "initialisationGestionnaire", e); //$NON-NLS-1$//$NON-NLS-2$
		}
		catch (final IOException e) {
			Serveur.logServeur.logMessage ("Erreur E/S lors de la mise en service du socket"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "initialisationGestionnaire", e); //$NON-NLS-1$//$NON-NLS-2$
		}
		catch (final NullPointerException e) {
			Serveur.logServeur.logMessage ("Ajout non authorise d'un etat"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "initialisationGestionnaire", e); //$NON-NLS-1$//$NON-NLS-2$
		}
		catch (final ClassCastException e) {
			Serveur.logServeur.logMessage ("Ajout d'un etat qui existe deja"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "initialisationGestionnaire", e); //$NON-NLS-1$//$NON-NLS-2$
		}
	}

	/**
	 * <p>
	 * Lister tout les canaux du serveur pour avoir leur nombre d'utilisateur et
	 * leur sujet.
	 * </p>
	 * 
	 * @param nomCanal
	 *            Nom du canal sur lequel on veux connaître les informations (si
	 *            le paramètre est vide tout les canaux seront choisi)
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void lister (final String nomCanal) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			try {
				reponse = "+OK_LISTER" + GestionnaireConnexion.CR + this.serveur.listerCanaux (nomCanal); //$NON-NLS-1$
			}
			catch (final CanalInexistantException e) {
				reponse = "-ERR_CANALINTROUVABLE " + nomCanal; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Liste les utilisateurs d'un canal.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal dont on veut connaître les membres.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void membre (final String nom_canal) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			final int tmp = this.serveur.canalExistant (nom_canal);

			if (tmp != -1) {
				reponse = "+OK_MEMBRE " + nom_canal + GestionnaireConnexion.CR + this.listeCanaux.get (nom_canal).listerMembre (); //$NON-NLS-1$
			}
			else {
				reponse = "-ERR_CANALINTROUVABLE " + nom_canal; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Envoyer un message sur un canal.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal sur lequel envoyer le message.
	 * @param message
	 *            Message à envoyer sur le canal.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void message (final String nom_canal, final String message) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie) && this.etats.contains (Etat.connexion_canal)) {
			if (this.serveur.canalExistant (nom_canal) != -1) {
				if (this.listeCanaux.containsKey (nom_canal)) {
					this.listeCanaux.get (nom_canal).notifier (Evenement.PARLE, this.client.getLogin (), message);
					reponse = "+OK_MESSAGE"; //$NON-NLS-1$
				}
				else {
					reponse = "-ERR_NONCONNECTE"; //$NON-NLS-1$
				}
			}
			else {
				reponse = "-ERR_CANALINTROUVABLE"; //$NON-NLS-1$
			}
			reponse += " " + nom_canal; //$NON-NLS-1$
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}
	
	/**
	 * <p>
	 * Connaître le message du jour
	 * </p>
	 * 
	 * @throws IOException
	 * 			Erreur d'entrée sortie.
	 */
	private synchronized void motd () throws IOException {
		String reponse = this.serveur.requeteMOTD ();
		
		if (reponse.compareTo ("") == 0) { //$NON-NLS-1$
			reponse = "-ERR_INTERNE"; //$NON-NLS-1$
		}
		else {
			reponse = "+OK_MOTD \"" + reponse + "\""; //$NON-NLS-1$ //$NON-NLS-2$
		}
		this.envoyer (reponse);
	}
	
	/**
	 * <p>
	 * Informations envoyer par le serveur afin de mettre au courant tout les
	 * utilisateurs d'un canal ce qui s'y passe dessus.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal sur lequel envoyer les informations.
	 * @param evenement
	 *            Evenement correspondant à l'information envoyé.
	 * @param login
	 *            Login de l'utilisateur originaire de l'information.
	 * @param message
	 *            Message à envoyer sur le canal.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public void notifier (final String nom_canal, final Evenement evenement, final String login, final String message) throws IOException {
		String res = "NOTIFIER " + nom_canal + " " + evenement.toString (); //$NON-NLS-1$ //$NON-NLS-2$

		switch (evenement) {
			case REJOINT:
			case PARTI:
			case EJECTE:
				res += " " + login; //$NON-NLS-1$
				break;
			case BANNI:
				res += " " + login; //$NON-NLS-1$
				break;
			case PARLE:
				res += " " + login + " \"" + message + "\""; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
				break;

			case SUJETCHANGE:
				res += " " + "\"" + message + "\""; //$NON-NLS-1$ //$NON-NLS-2$//$NON-NLS-3$
				break;

			case SUPPRIME:
				break;
		}
		this.envoyer (res);
	}

	/**
	 * <p>
	 * Quitter le serveur.
	 * </p>
	 * 
	 * @param msg
	 *            Message de départ (non obligatoire).
	 */
	public synchronized void quitter (final String msg) {
		// Départ des canaux auxquels on est connecté
		if (this.etats.contains (Etat.connexion_canal)) {
			for (final Entry<String, Canal> entry:this.listeCanaux.entrySet ()) {
				final Canal valeur = entry.getValue ();

				try {
					valeur.notifier (Evenement.PARLE, this.client.getLogin (), msg);
					valeur.enleverGC (this);
				}
				catch (final NonConnecteException e) {
					Serveur.logCoDeco.logMessage (this.client.getLogin () + " n'est pas connecté au canal"); //$NON-NLS-1$
					Serveur.logCoDeco.logException ("GestionnaireException", "quitter", e); //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (final IOException e) {
					Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
					Serveur.logServeur.logException ("GestionnaireException", "quitter", e); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}

		// Suppression des connexions directes
		if (this.etats.contains (Etat.connexion_directe)) {
			for (int i = 0;i < this.listeNomAccepte.size ();i++) {
				GestionnaireConnexion gc = new GestionnaireConnexion ();

				try {
					gc = this.serveur.connexion_directe (this.listeNomAccepte.get (i));

					gc.listeNomAccepte.remove (this.client.getLogin ());
					gc = null;
				}
				catch (final LoginIntrouvableException e) {
					Serveur.logCoDeco.logMessage (this.client.getLogin () + " n'est pas connecté"); //$NON-NLS-1$
					Serveur.logCoDeco.logException ("GestionnaireException", "quitter", e); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}

		// Deconnexion du serveur et remise à zero du GC
		if (this.serveur != null) {
			this.serveur.deconnexion (this.numeroGC);
		}
	}

	/**
	 * <p>
	 * Quitter un canal.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du vanal qu'on veut quitter.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void quitterCanal (final String nom_canal) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie) && this.etats.contains (Etat.connexion_canal)) {
			final int tmp = this.serveur.canalExistant (nom_canal);

			if (tmp != -1) {
				if (!this.listeCanaux.containsKey (nom_canal)) {
					reponse = "-ERR_NONCONNECTE " + nom_canal; //$NON-NLS-1$
				}
				else {
					try {
						this.listeCanaux.get (nom_canal).enleverGC (this);

						reponse = "+OK_QUITTERCANAL " + nom_canal; //$NON-NLS-1$
					}
					catch (final NonConnecteException e) {
						reponse = "-ERR_NONCONNECTE " + nom_canal; //$NON-NLS-1$
					}
					this.enleverCanal (nom_canal);
					if (this.listeCanaux.isEmpty ()) {
						this.etats.remove (Etat.connexion_canal);
					}
				}
			}
			else {
				reponse = "-ERR_CANALINTROUVABLE " + nom_canal; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Rejoindre un canal.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal que l'on veut rejoindre.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void rejoindre (final String nom_canal) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			final int tmp = this.serveur.canalExistant (nom_canal);

			if (tmp != -1) {
				try {
					this.listeCanaux.put (nom_canal, this.serveur.getListeCanaux ().get (tmp));
					this.listeCanaux.get (nom_canal).ajouterGC (this);
					if (!this.etats.contains (Etat.connexion_canal)) {
						this.etats.add (Etat.connexion_canal);
					}

					reponse = "+OK_REJOINDRE " + nom_canal; //$NON-NLS-1$
				}
				catch (final DejaConnecteException e) {
					reponse = "-ERR_DEJACONNECTE " + nom_canal; //$NON-NLS-1$
				}
			}
			else {
				reponse = "-ERR_CANALINTROUVABLE " + nom_canal; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Méthode principal de lancement du thread.
	 * </p>
	 */
	@Override
	public void run () {
		String message = ""; //$NON-NLS-1$
		List<String> retour = Collections.synchronizedList (new ArrayList<String> ());

		/*try {
			this.envoyer (Serveur.MOTD);
			this.sortie.flush ();
		}
		catch (final IOException e) {
			Serveur.logServeur.logMessage ("Erreur d'E/S lors de l'utilisation du socket"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "run", e); //$NON-NLS-1$ //$NON-NLS-2$
		}*/
		while (this.lance) {
			boolean err_syntaxe = false;
			String cmd = ""; //$NON-NLS-1$
			int size = 0;

			try {
				message = this.entree.readLine ();
				if (message != null) {
					retour = this.analyseMessage (message);
				}
			}
			catch (final IOException e1) {
				if (this.lance) {
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
				}
				break;
			}
			catch (final CommandeIncorrecteException e) {
				try {
					this.envoyer ("-ERR_SYNTAXE"); //$NON-NLS-1$
				}
				catch (final IOException e1) {
					Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
					Serveur.logServeur.logException ("GestionnaireConnexion", "run", e); //$NON-NLS-1$ //$NON-NLS-2$
					break;
				}
				continue;
			}
			size = retour.size ();
			if (size != 0) {
				cmd = retour.get (0);
			}
			else {
				break;
			}

			/* Commandes enregistrement de la connexion */
			if (0 == cmd.compareToIgnoreCase ("IDENTIFIER")) { //$NON-NLS-1$
				if (size == 3) {
					try {
						this.identifier (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("QUITTER")) { //$NON-NLS-1$
				if (size == 1) {
					this.quitter (""); //$NON-NLS-1$
					this.lance = false;
				}
				else if (size == 2) {
					this.quitter (retour.get (1));
					this.lance = false;
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("REJOINDRE")) { //$NON-NLS-1$		
				if (size == 2) {
					try {
						this.rejoindre (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("MESSAGE")) { //$NON-NLS-1$
				if (size == 3) {
					try {
						this.message (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("QUITTERCANAL")) { //$NON-NLS-1$		
				if (size == 2) {
					try {
						this.quitterCanal (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("MEMBRE")) { //$NON-NLS-1$		
				if (size == 2) {
					try {
						this.membre (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}

			/* Commandes d'administration */
			else if (0 == cmd.compareToIgnoreCase ("CNL_CREER")) { //$NON-NLS-1$
				if (size == 3) {
					try {
						this.cnl_creer (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("CNL_SUPPRIMER")) { //$NON-NLS-1$
				if (size == 2) {
					try {
						this.cnl_supprimer (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("CNL_CHANGERSUJET")) { //$NON-NLS-1$
				if (size == 3) {
					try {
						this.cnl_changerSujet (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("UTL_EJECTER")) { //$NON-NLS-1$
				if (size == 3) {
					try {
						this.utl_ejecter (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("SRV_RELANCER")) { //$NON-NLS-1$
				try {
					if (size == 2) {
						this.srv_relancer (retour.get (1));
					}
					else if (retour.size () == 1) {
						this.srv_relancer (""); //$NON-NLS-1$
					}
					else {
						err_syntaxe = true;
					}
				}
				catch (final IOException e) {
					Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					break;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("SRV_STOPPER")) { //$NON-NLS-1$
				try {
					if (size == 2) {
						this.srv_stopper (retour.get (1));
					}
					else if (size == 1) {
						this.srv_stopper (""); //$NON-NLS-1$
					}
					else {
						err_syntaxe = true;
					}
				}
				catch (final IOException e) {
					Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					break;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("SRV_LISTER")) { //$NON-NLS-1$
				if (size == 1) {
					try {
						this.srv_lister ();
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("UTL_BANNIR")) { //$NON-NLS-1$
				if (size == 2) {
					try {
						this.utl_bannir (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("LISTER")) { //$NON-NLS-1$1 
				try {
					if (size == 2) {
						this.lister (retour.get (1));
					}
					else if (size == 1) {
						this.lister (""); //$NON-NLS-1$
					}
					else {
						err_syntaxe = true;
					}
				}
				catch (final IOException e) {
					Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					break;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("INFOMEMBRE")) { //$NON-NLS-1$
				if (retour.size () == 2) {
					try {
						this.infoMembre (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("MOTD")) { //$NON-NLS-1$
				if (retour.size () == 1) {
					try {
						this.motd ();
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("CONNEXIONDIRECTE")) { //$NON-NLS-1$
				if (retour.size () == 3) {
					try {
						this.connexionDirecte (retour.get (1), retour.get (2));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("+OK_CONNEXIONDIRECTE")) { //$NON-NLS-1$
				if (retour.size () == 4) {
					try {
						this.connexionDirecteAccepte (retour.get (1), retour.get (2), retour.get (3));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else if (0 == cmd.compareToIgnoreCase ("+KO_CONNEXIONDIRECTE")) { //$NON-NLS-1$
				if (retour.size () == 2) {
					try {
						this.connexionDirecteRefuse (retour.get (1));
					}
					catch (final IOException e) {
						Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
						break;
					}
				}
				else {
					err_syntaxe = true;
				}
			}
			else {
				try {
					this.envoyer ("-ERR_CMDINCONNUE " + cmd); //$NON-NLS-1$
				}
				catch (final IOException e) {
					Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					break;
				}
			}
			if (err_syntaxe) {
				try {
					this.envoyer ("-ERR_SYNTAXE"); //$NON-NLS-1$
					this.sortie.flush ();
				}
				catch (final IOException e) {
					Serveur.logCoDeco.logMessage ("Déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					this.quitter ("déconnexion sauvage de " + this.client.getLogin ()); //$NON-NLS-1$
					break;
				}
			}
		}
	}

	/**
	 * <p>
	 * Arrêter ou démarrer un thread.
	 * </p>
	 * 
	 * @param lance1
	 *            Boolean correspondant à l'état du thread.
	 */
	public void setLance (final boolean lance1) {
		this.lance = lance1;
	}

	/**
	 * <p>
	 * Lister les clients du serveur.
	 * </p>
	 * 
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void srv_lister () throws IOException {
		String rep = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				rep = "+OK_SRV_LISTER" + GestionnaireConnexion.CR + this.serveur.listerServeur (); //$NON-NLS-1$

				this.envoyer (rep);
			}
			else {
				this.envoyer ("-ERR_NONADMINISTRATEUR"); //$NON-NLS-1$
			}
		}
		else {
			this.envoyer ("-ERR_ETAT"); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Relancer le serveur.
	 * </p>
	 * 
	 * @param s
	 *            Message qui sera envoyé à tout les clients du serveur
	 *            (optionnel).
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void srv_relancer (final String s) throws IOException {
		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				this.serveur.relancerServeur (s);
			}
			else {
				this.envoyer ("-ERR_NONADMINISTRATEUR"); //$NON-NLS-1$
			}
		}
		else {
			this.envoyer ("-ERR_ETAT"); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Arrêt du serveur.
	 * </p>
	 * 
	 * @param s
	 *            Message qui sera envoyé à tout les clients du serveur
	 *            (optionnel).
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void srv_stopper (final String s) throws IOException {
		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				this.serveur.stopperServeur (s);
			}
			else {
				this.envoyer ("-ERR_NONADMINISTRATEUR"); //$NON-NLS-1$
			}
		}
		else {
			this.envoyer ("-ERR_ETAT"); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Message envoyé par le serveur pour une coupure de connexion. Lors d'un
	 * relancement ou de l'arrêt.
	 * </p>
	 * 
	 * @param message
	 *            Message qui sera envoyer lors de la coupure de la connexion
	 *            (optionnel).
	 */
	public void stopper (final String message) {
		try {
			if (message.equals ("")) { //$NON-NLS-1$
				this.envoyer ("FINCONNEXION"); //$NON-NLS-1$
			}
			else {
				this.envoyer ("FINCONNEXION " + message); //$NON-NLS-1$
			}
		}
		catch (final IOException e) {
			Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "run", e); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this.vider ();
	}

	/**
	 * <p>
	 * Bannir un utilisateur du serveur.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur à bannir.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public synchronized void utl_bannir (final String login) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				try {
					this.serveur.bannirUtilisateur (login);

					reponse = "+OK_UTL_BANNIR " + login; //$NON-NLS-1$
				}
				catch (final LoginIntrouvableException e1) {
					reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
				}
				catch (NonConnecteException e) {
					reponse = "-ERR_NONCONNECTE " + login; //$NON-NLS-1$
				}
			}
			else {
				reponse = "-ERR_NONADMINISTRATEUR"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Ejecter un utilisateur du canal.
	 * </p>
	 * 
	 * @param login
	 *            Login du client à éjecter.
	 * @param nom_canal
	 *            Nom du canal ou se trouve le client à éjecter.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	private synchronized void utl_ejecter (final String login, final String nom_canal) throws IOException {
		String reponse = ""; //$NON-NLS-1$

		if (this.etats.contains (Etat.authentifie)) {
			if (this.client.estAdministrateur ()) {
				try {
					this.serveur.ejecterUtilisateur (login, nom_canal);

					reponse = "+OK_UTL_EJECTER " + login + " " + nom_canal; //$NON-NLS-1$ //$NON-NLS-2$
				}
				catch (final CanalInexistantException e) {
					reponse = "-ERR_CANALINTROUVABLE " + nom_canal; //$NON-NLS-1$
				}
				catch (final LoginIntrouvableException e1) {
					reponse = "-ERR_LOGININTROUVABLE " + login; //$NON-NLS-1$
				}
				catch (final NonConnecteException e2) {
					reponse = "-ERR_NONCONNECTE " + login + " " + nom_canal; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			else {
				reponse = "-ERR_NONADMINISTRATEUR"; //$NON-NLS-1$
			}
		}
		else {
			reponse = "-ERR_ETAT"; //$NON-NLS-1$
		}
		this.envoyer (reponse);
	}

	/**
	 * <p>
	 * Vidage du gestionnaire de connexion dans l'optique d'une possible
	 * réutilisation.
	 * </p>
	 */
	public synchronized void vider () {
		try {
			this.sortie.close ();
			this.entree.close ();
			this.socket.close ();
		}
		catch (final IOException e) {
			Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
			Serveur.logServeur.logException ("GestionnaireConnexion", "run", e); //$NON-NLS-1$ //$NON-NLS-2$
		}

		this.socket = null;
		this.serveur = null;

		this.client.vider ();
		this.etats.clear ();
		this.listeCanaux.clear ();

		this.numeroGC = 0;
		this.lance = false;
		this.sortie = null;
		this.entree = null;
	}
}
