/*
 *  PCUServeur for the PCU project.
 *  Copyright (C) 2010  PLU Julien
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  For questions: julien.plu@redaction-developpez.com
 */
package pcuserveur.lib;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

/**
 * @author PLU Julien
 * @version 1.0 
 * 
 * <b> 
 *     Cette classe permet de faire des log contenant les erreurs
 *     et les messages généré par le serveur.
 * </b>
 * <p>
 *     Un MonLog est caractérisé par les informations suivantes:
 *     <ul>
 *         <li>Une DateFormat pour la gestion des fichiers.</li>
 *         <li>Un Handler pour la manipulation de fichiers.</li>
 *         <li>Un Logger pour la gestion de l'écriture.</li>
 *     </ul>
 * </p>
 * 
 * @see GestionnaireConnexion
 * @see Serveur
 * @see Canal
 */
public class MonLog {
	/**
	 * <p>
	 * Un handler pour la gestion des fichiers.
	 * </p>
	 */
	private Handler descripteur;
	/**
	 * <p>
	 * Une date pour connaître le jour des messages placés dans les logs.
	 * </p>
	 */
	private final DateFormat formatDate = new SimpleDateFormat ("dd-MM-yyyy"); //$NON-NLS-1$
	/**
	 * <p>
	 * Un logger pour l'écriture des logs dans les fichiers correspondants.
	 * </p>
	 */
	private Logger log;

	/**
	 * <p>
	 * Constructeur par défaut.
	 * </p>
	 */
	public MonLog () {
		// Constructeur par défaut.
	}

	/**
	 * <p>
	 * Construcuteur paramétré.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du fichier créé.
	 */
	public MonLog (final String nom) {
		this.log = Logger.getLogger ("pcuserveur.lib.MonLog" + nom); //$NON-NLS-1$

		this.log.setLevel (Level.ALL);

		final String date = this.formatDate.format (new Date ());

		try {
			final File logFile = new File ("." + File.separator + "Logs" + File.separator + date + "." + nom + ".log"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

			if (!logFile.createNewFile ()) {
				this.descripteur = new FileHandler (logFile.getAbsolutePath (), true);
			}
			else {
				this.descripteur = new FileHandler (logFile.getAbsolutePath ());
			}

		}
		catch (final SecurityException e) {
			Serveur.logServeur.logMessage ("Erreur de sécurité"); //$NON-NLS-1$
			Serveur.logServeur.logException ("MonLog", "MonLog", e); //$NON-NLS-1$ //$NON-NLS-2$
		}
		catch (final IOException e) {
			Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
			Serveur.logServeur.logException ("MonLog", "MonLog", e); //$NON-NLS-1$ //$NON-NLS-2$
		}
		this.descripteur.setFormatter (new SimpleFormatter ());
		this.log.addHandler (this.descripteur);
	}

	/**
	 * <p>
	 * Cette méthode permet de fermer proprement le fichier de log.
	 * </p>
	 */
	public void fermer () {
		this.descripteur.close ();
	}

	/**
	 * <p>
	 * Cette méthode log une nouvelle exception.
	 * </p>
	 * 
	 * @param classe
	 *            La classe qui a générrer l'exception.
	 * @param methode
	 *            La méthode dans la classe qui a générer l'exception.
	 * @param exception
	 *            L'exception qui a été levée.
	 */
	public void logException (final String classe, final String methode, final Throwable exception) {
		this.log.throwing (classe, methode, exception);
	}

	/**
	 * <p>
	 * Cette méthode log un nouveau message.
	 * </p>
	 * 
	 * @param message
	 *            Le message à écrire dans le log.
	 */
	public void logMessage (final String message) {
		this.log.info (message);
	}
}
