/*
 *  PCUServeur for the PCU project.
 *  Copyright (C) 2010  PLU Julien
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  For questions: julien.plu@redaction-developpez.com
 */
package pcuserveur.lib;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.NetworkInterface;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;

/**
 * @author PLU Julien
 * @version 1.0 
 * 
 * <b>
 *     Cette classe représente le serveur en lui même.
 * </b>
 * <p>
 *     Un Serveur est caractérisé par les informations suivantes:
 *     <ul>
 *         <li>Un entier pour le port par défaut du serveur.</li>
 *         <li>Un entier pour le nombre maximum de connexion.</li>
 *         <li>Un entier pour le nombre maximum de canaux.</li>
 *         <li>Un String pour le message du jour.</li>
 *         <li>Un String pour le nombre maximum de connexion.</li>
 *         <li>Un String pour bannir le client.</li>
 *         <li>Une instance de MonLog pour les canaux.</li>
 *         <li>Une instance de MonLog pour les connexion/déconnexion.</li>
 *         <li>Une instance de MonLog pour le serveur.</li>
 *         <li>La connexion à la base de donnée.</li>
 *         <li>Un boolean pour savoir si le serveur est lancé.</li>
 *         <li>Un ArrayList contenant les gestionnaire de connexion.</li>
 *         <li>Un ArrayList contenant les canaux.</li>
 *         <li>Un entier pour le nombre de canaux du serveur.</li>
 *         <li>Un entier pour le nombre de gestionnaire de connexion.</li>
 *         <li>La socket du serveur.</li>
 *     </ul>
 * </p>
 * 
 * @see Canal
 * @see GestionnaireConnexion
 * @see CanalExistantException
 * @see CanalInexistantException
 * @see Evenement
 * @see MonLog
 * @see IdentifierException
 * @see NbCanauxMaxException
 */
public class Serveur {
	/**
	 * <p>
	 * Nombre maximum de canaux que le serveur pourra héberger.
	 * </p>
	 * 
	 * @see Serveur#Serveur ()
	 * @see Serveur#initialiser ()
	 * @see Serveur#configuration (int, int)
	 */
	private static int CANAUX_MAX;
	/**
	 * <p>
	 * Nombre maximum de connexion possible sur le serveur.
	 * </p>
	 * 
	 * @see Serveur#Serveur ()
	 * @see Serveur#initialiser ()
	 * @see Serveur#configuration (int, int)
	 */
	private static int CONNEXION_MAX;
	/**
	 * <p>
	 * Une instance de MyLogger pour les canaux.
	 * </p>
	 */
	public static MonLog logCanaux;
	/**
	 * <p>
	 * Une instance de MyLogger pour les connexions/déconnexions.
	 * </p>
	 */
	public static MonLog logCoDeco;
	/**
	 * <p>
	 * Une instance de MyLogger pour le serveur.
	 * </p>
	 */
	public static MonLog logServeur;
	/**
	 * <p>
	 * Mot de passe de l'utilisateur de la BDD.
	 * </p>
	 * 
	 * @see Serveur#Connexion ()
	 * @see Serveur#configuration (int, int)
	 */
	private static String MDP_BDD;
	/**
	 * <p>
	 * Message envoyé par le serveur lors de la connexion d'un client.
	 * </p>
	 * 
	 * @see GestionnaireConnexion#run ()
	 * @see Serveur#configuration (int, int)
	 */
	public static String MOTD;
	/**
	 * <p>
	 * Message envoyé par le serveur lorsque la nombre maximum de client est
	 * atteint.
	 * </p>
	 * 
	 * @see Serveur#configuration (int, int)
	 * @see Serveur#lancer ()
	 */
	private static String MSG_CONNEXION_MAX;
	/**
	 * <p>
	 * Message envoyé par le serveur lorsqu'un client est banni.
	 * </p>
	 * 
	 * @see Serveur#configuration (int, int)
	 * @see Serveur#bannirUtilisateur (String)
	 */
	private static String MSG_UTL_BANNI;
	/**
	 * <p>
	 * Nom de la BDD.
	 * </p>
	 * 
	 * @see Serveur#Connexion ()
	 * @see Serveur#configuration (int, int)
	 */
	private static String NOM_BDD;
	/**
	 * <p>
	 * Port par défaut sur lequel se connectera le serveur.
	 * </p>
	 * 
	 * @see Serveur#Serveur ()
	 * @see Serveur#configuration (int, int)
	 */
	private static int PORT_DEFAUT;
	/**
	 * <p>
	 * Port par défaut sur lequel écoute le serveur de la BDD.
	 * </p>
	 * 
	 * @see Serveur#Connexion ()
	 * @see Serveur#configuration (int, int)
	 */
	private static int PORT_SERVEUR_BDD;
	/**
	 * <p>
	 * Adresse du serveur de la BDD.
	 * </p>
	 * 
	 * @see Serveur#Connexion ()
	 * @see Serveur#configuration (int, int)
	 */
	private static String SERVEUR_BDD;
	/**
	 * <p>
	 * Utilisateur de la BDD.
	 * </p>
	 * 
	 * @see Serveur#Connexion ()
	 * @see Serveur#configuration (int, int)
	 */
	private static String USER_BDD;
	/**
	 * <p>
	 * Connexion et gestion de la BDD.
	 * </p>
	 * 
	 * @see Serveur#connexion ()
	 * @see Serveur#initialiser ()
	 * @see Serveur#stopperServeur (String)
	 */
	private Connection conn;
	/**
	 * <p>
	 * Lancement ou arrêt du serveur.
	 * </p>
	 * 
	 * @see Serveur#initialiser ()
	 * @see Serveur#lancer ()
	 * @see Serveur#stopperServeur (String)
	 */
	private boolean lance = false;
	/**
	 * <p>
	 * Liste des canaux contenus sur le serveur.
	 * </p>
	 * 
	 * @see Serveur#Serveur ()
	 * @see Serveur#canalExistant (String)
	 * @see Serveur#changerSujetCanal (String, String)
	 * @see Serveur#creerCanal (String, String)
	 * @see Serveur#ejecterUtilisateur (String, String)
	 * @see Serveur#initialiser ()
	 * @see Serveur#listerCanaux (String)
	 * @see Serveur#listerMembres (String)
	 * @see Serveur#relancerServeur (String)
	 * @see Serveur#stopperServeur (String)
	 * @see Serveur#supprimerCanal (String)
	 */
	private ArrayList<Canal> listeCanaux = new ArrayList<Canal> ();
	/**
	 * <p>
	 * Liste des gestionnaire de connexion contenus sur le serveur.
	 * </p>
	 * 
	 * @see Serveur#Serveur ()
	 * @see Serveur#ajouterGC (Socket)
	 * @see Serveur#chercherUtilisateur (String)
	 * @see Serveur#deconnexion (int)
	 * @see Serveur#ejecterUtilisateur (String, String)
	 * @see Serveur#informationMembre (String)
	 * @see Serveur#initialiser ()
	 * @see Serveur#listerServeur ()
	 * @see Serveur#relancerServeur (String)
	 * @see Serveur#stopperServeur (String)
	 */
	private ArrayList<GestionnaireConnexion> listeGestionnaire = new ArrayList<GestionnaireConnexion> ();
	/**
	 * <p>
	 * Nombre de canaux contenus sur le serveur.
	 * </p>
	 * 
	 * @see Serveur#creerCanal (String, String)
	 * @see Serveur#initialiser ()
	 * @see Serveur#stopperServeur (String)
	 * @see Serveur#supprimerCanal (String)
	 */
	private int nbCanaux;

	/**
	 * <p>
	 * Nombre de gestionnaire de connexion contenus sur le serveur.
	 * </p>
	 * 
	 * @see Serveur#ajouterGC (Socket)
	 * @see Serveur#deconnexion (int)
	 * @see Serveur#initialiser ()
	 * @see Serveur#listerServeur ()
	 * @see Serveur#stopperServeur (String)
	 */
	private int nbGC;

	/**
	 * <p>
	 * Socket sur lesquels les gestionnaires de connexions seront connectés.
	 * </p>
	 * 
	 * @see Serveur#initialiser ()
	 * @see Serveur#lancer ()
	 * @see Serveur#stopperServeur (String)
	 */
	private final ServerSocket socket;

	/**
	 * <p>
	 * Constructeur par défaut.
	 * </p>
	 * 
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 * @throws RuntimeException
	 * 				Erreur d'exécution.
	 */
	public Serveur () throws IOException, RuntimeException {
		Serveur.configuration (this.nbGC, this.nbCanaux);
		this.socket = new ServerSocket (Serveur.PORT_DEFAUT);

		for (int i = 0;i < Serveur.CONNEXION_MAX;i++) {
			this.listeGestionnaire.add (new GestionnaireConnexion ());
		}
		for (int i = 0;i < Serveur.CANAUX_MAX;i++) {
			this.listeCanaux.add (new Canal ());
		}
	}

	/**
	 * <p>
	 * Créer et ajouter un GestionnaireConnexion à partir d'un socket dans la
	 * listes des gestionnaires.
	 * </p>
	 * 
	 * @param socket1
	 *            Socket du gestionnaire de connexion à ajouter.
	 */
	public void ajouterGC (final Socket socket1) {
		this.nbGC++;
		final int i = this.premierNull (this.listeGestionnaire);

		if (i != -1) {
			this.listeGestionnaire.get (i).initialisationGestionnaire (socket1, this, i);
			new Thread (this.listeGestionnaire.get (i)).start ();
		}
	}

	/**
	 * <p>
	 * Bannir un client du serveur.
	 * </p>
	 * 
	 * @param login
	 *          Login du client à bannir.
	 *            
	 * @throws LoginIntrouvableException
	 *          Le client n'existe pas.
	 * @throws IOException
	 * 			Erreur d'entrée sortie.
	 * @throws NonConnecteException
	 * 			Utilisateur non connecté.
	 */
	public void bannirUtilisateur (final String login) throws LoginIntrouvableException, IOException, NonConnecteException {
		final int place = this.chercherUtilisateur (login);

		if (place != -1) {
			PreparedStatement state;

			try {
				state = this.conn.prepareStatement ("UPDATE utilisateurs SET active = '0' WHERE identifiant = ?"); //$NON-NLS-1$

				state.setString (1, login);
				state.executeUpdate ();
				this.listeGestionnaire.get (place).stopper (Serveur.MSG_UTL_BANNI);
				this.listeCanaux.get (place).notifier (Evenement.BANNI, login, ""); //$NON-NLS-1$
				this.listeCanaux.get (place).enleverGC (login);
				Serveur.logCoDeco.logMessage ("Le client " + login + " a été banni"); //$NON-NLS-1$ //$NON-NLS-2$
				state.close ();
			}
			catch (final SQLException e) {
				Serveur.logServeur.logException ("Serveur", "bannirUtilisateur", e); //$NON-NLS-1$//$NON-NLS-2$
			}
		}
		else {
			throw new LoginIntrouvableException (login + " n'est pas connecté"); //$NON-NLS-1$
		}
	}

	/**
	 * <p>
	 * Savoir si un canal existe sur le serveur.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal à rechercher.
	 * @return La position du canal -1 sinon.
	 */
	public int canalExistant (final String nom) {
		for (int i = 0;i < this.listeCanaux.size ();i++) {
			if ((this.listeCanaux.get (i).getNom () != null) && this.listeCanaux.get (i).getNom ().equals (nom)) {
				return i;
			}
		}

		return -1;
	}

	/**
	 * <p>
	 * Changer le sujet d'un canal.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal dont on veut changer le sujet.
	 * @param sujet
	 *            Nouveau sujet du canal.
	 * @throws CanalInexistantException
	 *             Le canal n'existe pas.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public void changerSujetCanal (final String nom, final String sujet) throws CanalInexistantException, IOException {
		final int tmp = this.canalExistant (nom);

		if (tmp != -1) {
			this.listeCanaux.get (tmp).changerSujet (sujet);
			Serveur.logCanaux.logMessage ("Le sujet du canal " + nom + " a été changé en " + sujet); //$NON-NLS-1$//$NON-NLS-2$
		}
		else {
			throw new CanalInexistantException ();
		}
	}

	/**
	 * <p>
	 * Savoir si un utilisateur existe sur le serveur.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur a rechercher.
	 * @return La position de l'utilisateur -1 sinon.
	 */
	private int chercherUtilisateur (final String login) {
		boolean trouve = false;
		int i = 0;

		while (i < this.listeGestionnaire.size ()) {
			if (!this.listeGestionnaire.get (i).getClient ().estVide ()) {
				if (this.listeGestionnaire.get (i).getClient ().getLogin ().equals (login)) {
					trouve = true;
					break;
				}
			}
			i++;
		}
		if (trouve) {
			return i;
		}

		return -1;
	}
	
	/**
	 * <p>
	 * Configuration du serveur par la lecture du fichier de configuration.
	 * </p>
	 * 
	 * @param nbCo
	 * 			Nombre d'utilisateur connecté sur le serveur.
	 * @param nbC
	 * 			Nombre de canaux créé sur le serveur.
	 * 
	 * @throws RuntimeException
	 *             Erreur lors de la configuration.
	 */
	public static void configuration (int nbCo, int nbC) throws RuntimeException {
		Serveur.logCanaux = new MonLog ("Canaux"); //$NON-NLS-1$
		Serveur.logServeur = new MonLog ("Serveur"); //$NON-NLS-1$
		Serveur.logCoDeco = new MonLog ("CoDeco"); //$NON-NLS-1$
		final File file = new File ("." + File.separator + "conf" + File.separator + "serveur.conf"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		try {
			final BufferedReader r = new BufferedReader (new InputStreamReader (new FileInputStream (file), "UTF-8")); //$NON-NLS-1$
			String contenu = r.readLine ();
			int i = 0;

			while (contenu != null) {
				if (contenu.charAt (0) == '#') {
					continue;
				}

				final String[] tab = contenu.split ("="); //$NON-NLS-1$

				if (tab[0].equals ("PORT_ECOUTE")) { //$NON-NLS-1$
					Serveur.PORT_DEFAUT = Integer.parseInt (tab[1]);
					i++;
				}
				else if (tab[0].equals ("CONNEXION_MAX")) { //$NON-NLS-1$
					Serveur.CONNEXION_MAX = Integer.parseInt (tab[1]);
					i++;
				}
				else if (tab[0].equals ("CANAUX_MAX")) { //$NON-NLS-1$
					Serveur.CANAUX_MAX = Integer.parseInt (tab[1]);
					i++;
				}
				else if (tab[0].equals ("MOTD")) { //$NON-NLS-1$
					Serveur.MOTD = ""; //$NON-NLS-1$
					
					if (tab[1].contains ("\\\\")) { //$NON-NLS-1$
						String phrase = tab[1];
						
						while (phrase.contains ("\\\\")) { //$NON-NLS-1$
							if (phrase.contains ("%d")) { //$NON-NLS-1$
								Date aujourdui = new Date ();
								DateFormat dateFormat = new SimpleDateFormat ("dd/MM/yyyy"); //$NON-NLS-1$
								
								phrase = phrase.replace ("%d", dateFormat.format (aujourdui)); //$NON-NLS-1$
							}
							if (phrase.contains ("%ip")) { //$NON-NLS-1$
								phrase = phrase.replace ("%ip", Serveur.getIp ()); //$NON-NLS-1$
							}
							if (phrase.contains ("%nc")) { //$NON-NLS-1$
								phrase = phrase.replace ("%nc", Integer.toString (nbC)); //$NON-NLS-1$
							}
							if (phrase.contains ("%nu")) { //$NON-NLS-1$
								phrase = phrase.replace ("%nu", Integer.toString (nbCo)); //$NON-NLS-1$
							}
							Serveur.MOTD += phrase.substring (1, phrase.length () - 2);
							phrase = r.readLine ();
						}
						if (phrase.contains ("%d")) { //$NON-NLS-1$
							Date aujourdui = new Date ();
							DateFormat dateFormat = new SimpleDateFormat ("dd/MM/yyyy"); //$NON-NLS-1$
							
							phrase = phrase.replace ("%d", dateFormat.format (aujourdui)); //$NON-NLS-1$
						}
						if (phrase.contains ("%ip")) { //$NON-NLS-1$
							phrase = phrase.replace ("%ip", Serveur.getIp ()); //$NON-NLS-1$
						}
						if (phrase.contains ("%nc")) { //$NON-NLS-1$
							phrase = phrase.replace ("%nc", Integer.toString (nbC)); //$NON-NLS-1$
						}
						if (phrase.contains ("%nu")) { //$NON-NLS-1$
							phrase = phrase.replace ("%nu", Integer.toString (nbCo)); //$NON-NLS-1$
						}
						Serveur.MOTD += phrase.substring (0, phrase.length () - 1);
					}
					else {
						Serveur.MOTD = tab[1].substring (1, tab[1].length () - 1);
					}
					
					i++;
				}
				else if (tab[0].equals ("MSG_CONNEXION_MAX")) { //$NON-NLS-1$
					Serveur.MSG_CONNEXION_MAX = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				else if (tab[0].equals ("MSG_UTL_BANNI")) { //$NON-NLS-1$
					Serveur.MSG_UTL_BANNI = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				else if (tab[0].equals ("SERVEUR_BDD")) { //$NON-NLS-1$
					Serveur.SERVEUR_BDD = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				else if (tab[0].equals ("PORT_SERVEUR_BDD")) { //$NON-NLS-1$
					Serveur.PORT_SERVEUR_BDD = Integer.parseInt (tab[1]);
					i++;
				}
				else if (tab[0].equals ("NOM_BDD")) { //$NON-NLS-1$
					Serveur.NOM_BDD = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				else if (tab[0].equals ("USER_BDD")) { //$NON-NLS-1$
					Serveur.USER_BDD = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				else if (tab[0].equals ("MDP_BDD")) { //$NON-NLS-1$
					Serveur.MDP_BDD = tab[1].substring (1, tab[1].length () - 1);
					i++;
				}
				contenu = r.readLine ();
			}
			r.close ();
			if (i != 11) {
				Serveur.logServeur.logMessage ("Une erreur est survenu avec le fichier de configuration, tout les paramètres n'ont pas été trouvés."); //$NON-NLS-1$
				Serveur.logServeur.logException ("Serveur", "configuration", new RuntimeException ()); //$NON-NLS-1$ //$NON-NLS-2$
				throw new RuntimeException ();
			}
		}
		catch (final UnsupportedEncodingException e) {
			Serveur.logServeur.logMessage ("Le fichier de configuration est introuvable."); //$NON-NLS-1$
			Serveur.logServeur.logException ("Serveur", "configuration", new RuntimeException ()); //$NON-NLS-1$ //$NON-NLS-2$
			throw new RuntimeException ();
		}
		catch (final FileNotFoundException e1) {
			Serveur.logServeur.logException ("Serveur", "configuration", e1); //$NON-NLS-1$ //$NON-NLS-2$
		}
		catch (final IOException e2) {
			Serveur.logServeur.logException ("Serveur", "configuration", e2); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/**
	 * <p>
	 * Connexion à la BDD.
	 * </p>
	 * 
	 * @throws ClassNotFoundException
	 *             Le pilote JDBC est introuvable.
	 */
	public void connexion () throws ClassNotFoundException {
		Class.forName ("com.mysql.jdbc.Driver"); //$NON-NLS-1$
		final String url = "jdbc:mysql://" + Serveur.SERVEUR_BDD + ":" + Serveur.PORT_SERVEUR_BDD + "/" + Serveur.NOM_BDD; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		final String utilisateur = Serveur.USER_BDD;
		final String mdp = Serveur.MDP_BDD;

		try {
			this.conn = DriverManager.getConnection (url, utilisateur, mdp);

			Serveur.logServeur.logMessage ("Connection bdd effective !"); //$NON-NLS-1$
		}
		catch (final SQLException e) {
			Serveur.logServeur.logMessage ("Connection bdd echoué"); //$NON-NLS-1$
			Serveur.logServeur.logException ("Serveur", "connexion", e); //$NON-NLS-1$ //$NON-NLS-2$
			throw new RuntimeException ();
		}
	}

	/**
	 * <p>
	 * Demande de connexion directe avec un autre client.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @return Le gestionnaire de connexion du client.
	 * @throws LoginIntrouvableException
	 *             Le login n'existe pas.
	 */
	public GestionnaireConnexion connexion_directe (final String login) throws LoginIntrouvableException {
		final int position = this.chercherUtilisateur (login);

		if (position != -1) {
			return this.listeGestionnaire.get (position);
		}
		throw new LoginIntrouvableException ();
	}

	/**
	 * <p>
	 * Demande de connexion directe avec un autre client.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @return Le gestionnaire de connexion du client.
	 * @throws LoginIntrouvableException
	 *             Le login n'existe pas.
	 */
	public GestionnaireConnexion connexion_directe_accepte (final String login) throws LoginIntrouvableException {
		final int position = this.chercherUtilisateur (login);

		if (position != -1) {
			return this.listeGestionnaire.get (position);
		}
		throw new LoginIntrouvableException ();
	}

	/**
	 * <p>
	 * Refus de la connexion directe avec un autre client.
	 * </p>
	 * 
	 * @param login
	 *            Login de la personne avec laquelle on veut une connexion.
	 * @return Le gestionnaire de connexion du client.
	 * @throws LoginIntrouvableException
	 *             Le login n'existe pas.
	 */
	public GestionnaireConnexion connexion_directe_refuse (final String login) throws LoginIntrouvableException {
		final int position = this.chercherUtilisateur (login);

		if (position != -1) {
			return this.listeGestionnaire.get (position);
		}
		throw new LoginIntrouvableException ();
	}

	/**
	 * <p>
	 * Creer un canal sur le serveur.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal à creer.
	 * @param sujet
	 *            Sujet du canal à creer.
	 * @throws CanalExistantException
	 *             Le canal n'existe pas.
	 * @throws NbCanauxMaxException
	 *             Trop de canaux ont été créé.
	 */
	public void creerCanal (final String nom, final String sujet) throws CanalExistantException, NbCanauxMaxException {
		if (this.canalExistant (nom) != -1) {
			throw new CanalExistantException ();
		}
		if (this.nbCanaux >= Serveur.CANAUX_MAX) {
			throw new NbCanauxMaxException ();
		}
		final int i = this.premierNull (this.listeCanaux);
		
		this.listeCanaux.get (i).initialiser (nom, sujet);
		this.nbCanaux++;
		Serveur.logServeur.logMessage ("Nouveau canal créé s'appellant " + nom + " et ayant pour sujet " + sujet); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * <p>
	 * Déconnexion d'un gestionnaire de connexion.
	 * </p>
	 * 
	 * @param num_gc
	 *            Emplacement du gestionnaire de connexion.
	 */
	public void deconnexion (final int num_gc) {
		Serveur.logCoDeco.logMessage ("Déconnexion de " + this.listeGestionnaire.get (num_gc).getClient ().getLogin ()); //$NON-NLS-1$
		this.listeGestionnaire.get (num_gc).vider ();
		this.nbGC--;
	}

	/**
	 * <p>
	 * Ejecte un utilisateur d'un canal.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur à éjecter.
	 * @param nom_canal
	 *            Nom du canal ou se situe l'utilisateur à éjecter.
	 * @throws CanalInexistantException
	 *             Le canal n'existe pas.
	 * @throws LoginIntrouvableException
	 *             Le login est introuvable.
	 * @throws NonConnecteException
	 *             L'utilisateur n'est pas connecté sur le canal.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public void ejecterUtilisateur (final String login, final String nom_canal) throws CanalInexistantException, LoginIntrouvableException, NonConnecteException, IOException {
		final int place = this.chercherUtilisateur (login);

		if (place != -1) {
			final int tmp = this.canalExistant (nom_canal);

			if (tmp != -1) {
				this.listeCanaux.get (tmp).notifier (Evenement.EJECTE, login, ""); //$NON-NLS-1$
				this.listeCanaux.get (tmp).enleverGC (login);
				this.listeGestionnaire.get (place).enleverCanal (nom_canal);
				Serveur.logCoDeco.logMessage (login + " a été éjecté du canal " + nom_canal); //$NON-NLS-1$
			}
			else {
				throw new CanalInexistantException ();
			}
		}
		else {
			throw new LoginIntrouvableException (login + " n'est pas connecté"); //$NON-NLS-1$
		}
	}
	
	/**
	 * <p>
	 * Méthode qui retourne l'adresse IP internet du poste sur lequel le serveur tourne.
	 * </p>
	 * 
	 * @return Adresse IP du serveur sur internet si le poste est connecté à internet
	 * 		   L'adresse sur le réseau locale sinon.
	 */
	public static String getIp () {
		String s = ""; //$NON-NLS-1$
		
		if (Serveur.isConnecteInternet ()) {
			URL u = null;
			InputStream i = null;
			
			try {
				u = new URL ("http://mon-ip.com/"); //$NON-NLS-1$
				i = u.openStream();
				
				BufferedReader b = new BufferedReader (new InputStreamReader (i));
				
				s = b.readLine();
				while (s != null) {
					if (s.contains ("Votre adresse IP est")) { //$NON-NLS-1$
						s = s.substring (65, 77);
						break;
					}
					s = b.readLine();
				}
			}
			catch (MalformedURLException e) {
				Serveur.logServeur.logMessage ("URL mal formée"); //$NON-NLS-1$
				Serveur.logServeur.logException ("Serveur", "getIp", e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			catch (IOException e) {
				Serveur.logServeur.logMessage ("Problème d'entrée sortie"); //$NON-NLS-1$
				Serveur.logServeur.logException ("Serveur", "getIp", e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			return s;
		}
		
		Enumeration<NetworkInterface> interfaces = null;
		
		try {
			interfaces = NetworkInterface.getNetworkInterfaces();
			int i = 0;
			
			while (interfaces.hasMoreElements ()) {
				NetworkInterface currentInterface = interfaces.nextElement ();
				Enumeration<InetAddress> addresses = currentInterface.getInetAddresses ();
				
				while (addresses.hasMoreElements ()) {
					i++;
					InetAddress currentAddress = addresses.nextElement ();
					
					if (i == 3) {
						s = currentAddress.getHostAddress ().toString ();
					}
				}
			}
		}
		catch (SocketException e) {
			Serveur.logServeur.logMessage ("Problème d'entrée sortie"); //$NON-NLS-1$
			Serveur.logServeur.logException ("Serveur", "getIp", e); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		return s;
	}

	/**
	 * <p>
	 * Renvoie la liste des canaux du serveur.
	 * </p>
	 * 
	 * @return La liste des canaux.
	 */
	public ArrayList<Canal> getListeCanaux () {
		return this.listeCanaux;
	}

	/**
	 * <p>
	 * Connaître les informations concernant un utilisateur du serveur.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur dont on veut connaître les information.
	 * @return Les information de l'utilisateur.
	 * @throws LoginIntrouvableException
	 *             Le login est introuvable.
	 */
	public String informationMembre (final String login) throws LoginIntrouvableException {
		final int i = this.chercherUtilisateur (login);

		if (i != -1) {
			return login + GestionnaireConnexion.CR + this.listeGestionnaire.get (i).infoCanaux ();
		}
		throw new LoginIntrouvableException (login + " n'est pas connecté"); //$NON-NLS-1$
	}

	/**
	 * <p>
	 * Initialise le serveur
	 * </p>
	 * 
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public void initialiser () throws IOException {
		this.socket.setReuseAddress (true);
		this.nbGC = 0;
		this.nbCanaux = 0;
		this.listeGestionnaire = null;
		this.listeCanaux = null;
		this.listeGestionnaire = new ArrayList<GestionnaireConnexion> (Serveur.CONNEXION_MAX);
		this.listeCanaux = new ArrayList<Canal> (Serveur.CANAUX_MAX);
		this.conn = null;
		this.lance = false;

		for (int i = 0;i < Serveur.CONNEXION_MAX;i++) {
			this.listeGestionnaire.add (new GestionnaireConnexion ());
		}
		for (int i = 0;i < Serveur.CANAUX_MAX;i++) {
			this.listeCanaux.add (new Canal ());
		}
	}
	
	/**
	 * <p>
	 * Renvoi un booleen pour savoir si le poste est connecté ou non à internet.
	 * </p>
	 * 
	 * @return Si le poste est connecté ou non à internet.
	 */
	public static boolean isConnecteInternet ()
    {
		try {
			URL url = new URL ("http://www.google.com"); //$NON-NLS-1$
            HttpURLConnection urlConnect = (HttpURLConnection)url.openConnection ();
            Object objData = urlConnect.getContent ();
            objData.toString ();
        }
		catch (MalformedURLException e) {
			Serveur.logServeur.logMessage ("URL mal formée"); //$NON-NLS-1$
			Serveur.logServeur.logException ("Serveur", "isConnecteInternet", e); //$NON-NLS-1$ //$NON-NLS-2$
			
			return false;
		}
        catch (IOException e) {
        	Serveur.logServeur.logMessage ("Non connecté à internet"); //$NON-NLS-1$
			Serveur.logServeur.logException ("Serveur", "isConnecteInternet", e); //$NON-NLS-1$ //$NON-NLS-2$
			
            return false;
        }
            
        return true;
    }


	/**
	 * <p>
	 * Lance le serveur.
	 * </p>
	 * 
	 * @throws ClassNotFoundException
	 *             Classe introuvable.
	 * @throws RuntimeException
	 *             Erreur lors de l'exécution.
	 */
	public void lancer () throws ClassNotFoundException, RuntimeException {
		this.lance = true;
		Serveur.logServeur.logMessage ("Lancement du serveur"); //$NON-NLS-1$

		try {
			this.connexion ();
		}
		catch (final ClassNotFoundException e) {
			this.lance = false;
			throw e;
		}
		while (this.lance) {
			Socket clientSocket = null;

			try {
				clientSocket = this.socket.accept ();
				Serveur.configuration (this.nbGC, this.nbCanaux);

				Serveur.logCoDeco.logMessage ("Connexion de " + clientSocket.getInetAddress ().toString ().substring (1)); //$NON-NLS-1$
			}
			catch (final IOException e) {
				if (this.lance) {
					Serveur.logServeur.logMessage ("Connexion echoue sur le port: 8888"); //$NON-NLS-1$
					Serveur.logServeur.logException ("Serveur", "lancer", e); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			if (this.lance) {
				if (this.nbGC == Serveur.CONNEXION_MAX) {
					GestionnaireConnexion gc = new GestionnaireConnexion ();
					gc.initialisationGestionnaire (clientSocket, this, 0);
					final Thread t = new Thread (gc);
					if (t.isAlive ()) {
						Serveur.logCoDeco.logMessage ("Déconnexion de " + gc.getIp () + " à cause d'un trop grand nombre de connectés"); //$NON-NLS-1$ //$NON-NLS-2$
						gc.stopper (Serveur.MSG_CONNEXION_MAX);
						gc = null;
					}
				}
				else {
					this.ajouterGC (clientSocket);
				}
			}
		}
	}

	/**
	 * <p>
	 * Lister les canaux du serveur afin de connaître leur sujet et leur nombre
	 * d'utilisateur.
	 * </p>
	 * 
	 * @param nomCanal
	 *            Nom du canal à lister (optionnel).
	 * @return Les informations sur le(s) canal (ou canaux).
	 * @throws CanalInexistantException
	 *             Le canal n'existe pas.
	 */
	public String listerCanaux (final String nomCanal) throws CanalInexistantException {
		final StringBuffer res = new StringBuffer ();
		boolean trouve = false;
		int i = 0;

		if (!nomCanal.equals ("")) { //$NON-NLS-1$
			while (i < this.listeCanaux.size ()) {
				if (!this.listeCanaux.get (i).estVide ()) {
					if (this.listeCanaux.get (i).getNom ().equals (nomCanal)) {
						trouve = true;
						break;
					}
				}
				i++;
			}
			if (trouve) {
				res.append (this.listeCanaux.get (i).getSignature ());
			}
			else {
				throw new CanalInexistantException ();
			}
		}
		else {
			int k = 0;

			for (int j = 0;j < this.listeCanaux.size ();j++) {
				if (!this.listeCanaux.get (j).estVide ()) {
					res.append (this.listeCanaux.get (j).getSignature ());
					if (k != this.nbCanaux - 1) {
						res.append (GestionnaireConnexion.CR);
					}
					k++;
				}
			}
		}

		return res.toString ();
	}

	/**
	 * <p>
	 * Lister les membres d'un canal.
	 * </p>
	 * 
	 * @param nom_canal
	 *            Nom du canal ou se situe les membres.
	 * @return Les informations sur les membres du canal.
	 */
	public String listerMembres (final String nom_canal) {
		final Canal canal = this.listeCanaux.get (this.canalExistant (nom_canal));

		return canal.listerMembre ();
	}

	/**
	 * <p>
	 * Liste les utilisateurs du serveur.
	 * </p>
	 * 
	 * @return Les utilisateurs du serveur.
	 */
	public String listerServeur () {
		final StringBuffer res = new StringBuffer ();
		int k = 0;
		boolean isNull = false;

		for (int i = 0;i < this.listeGestionnaire.size ();i++) {
			if (!this.listeGestionnaire.get (i).estVide ()) {
				if (this.listeGestionnaire.get (i).getClient ().getLogin () != null) {
					isNull = true;
					if ((k != this.nbGC) && isNull && (k != 0)) {
						res.append (GestionnaireConnexion.CR);
					}
					res.append (this.listeGestionnaire.get (i).getClient ().getLogin () + " " + this.listeGestionnaire.get (i).getIp ()); //$NON-NLS-1$
				}

				k++;
			}
		}

		return res.toString ();
	}

	/**
	 * <p>
	 * Encrypte une chaîne de caractère avec un cryptage MD5.
	 * </p>
	 * 
	 * @param key
	 *            La chaîne qui doit être encryptée.
	 *            
	 * @return La chaîne encryptée.
	 * 
	 * @throws NoSuchAlgorithmException
	 *             L'algorithme MD5 n'a pas été trouvé.
	 */
	private String md5 (final String key) throws NoSuchAlgorithmException {
		final byte[] uniqueKey = key.getBytes ();
		byte[] hash = null;
		hash = MessageDigest.getInstance ("MD5").digest (uniqueKey); //$NON-NLS-1$
		final StringBuffer hashString = new StringBuffer ();

		for (int i = 0;i < hash.length;++i) {
			final String hex = Integer.toHexString (hash[i]);

			if (hex.length () == 1) {
				hashString.append ('0');
				hashString.append (hex.charAt (hex.length () - 1));
			}
			else {
				hashString.append (hex.substring (hex.length () - 2));
			}
		}

		return hashString.toString ();
	}

	/**
	 * <p>
	 * Renvoie la première case vide d'une liste.
	 * </p>
	 * 
	 * @param liste
	 *            Liste dont ont veut connaître la première case vide.
	 * @return L'emplacement de la première case vide.
	 */
	private int premierNull (final ArrayList<? extends Vide> liste) {
		int res = -1;

		for (int i = 0;i < liste.size ();i++) {
			if (liste.get (i).estVide ()) {
				res = i;
				break;
			}
		}

		return res;
	}

	/**
	 * <p>
	 * Relancer le serveur.
	 * </p>
	 * 
	 * @param message
	 *            Le message à envoyer à tous les client avant le reboot du
	 *            serveur.
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 * @throws RuntimeException
	 *             Erreur d'exécution.
	 */
	public void relancerServeur (final String message) throws IOException, RuntimeException {
		for (int i = 0;i < this.listeGestionnaire.size ();i++) {
			if (!this.listeGestionnaire.get (i).estVide ()) {
				this.listeGestionnaire.get (i).setLance (false);
				this.listeGestionnaire.get (i).stopper (message);
			}
			this.listeGestionnaire.remove (i);
		}
		for (int i = 0;i < this.listeCanaux.size ();i++) {
			if (!this.listeCanaux.get (i).estVide ()) {
				this.listeCanaux.get (i).vider ();
			}
			this.listeCanaux.remove (i);
		}
		Serveur.logServeur.logMessage ("Relancement du serveur"); //$NON-NLS-1$
		this.initialiser ();
		try {
			this.lancer ();
		}
		catch (final ClassNotFoundException e) {
			Serveur.logServeur.logException ("Serveur", "relancerServeur", e); //$NON-NLS-1$ //$NON-NLS-2$
			throw new RuntimeException ();
		}
	}
	
	/**
	 * <p>
	 * Renvoi le MOTD
	 * </p>
	 * 
	 * @return Le MOTD
	 */
	public String requeteMOTD () {
		Serveur.configuration (this.nbGC, this.nbCanaux);
		
		return Serveur.MOTD;
	}

	/**
	 * <p>
	 * Arrête le serveur.
	 * </p>
	 * 
	 * @param message
	 *            Le message à envoyer aux clients, avant l'arrêt du serveur.
	 *            
	 * @throws IOException
	 *             Un envoie à un client a échoué.
	 */
	public void stopperServeur (final String message) throws IOException {
		for (int i = 0;i < this.listeGestionnaire.size ();i++) {
			if (!this.listeGestionnaire.get (i).estVide ()) {
				this.listeGestionnaire.get (i).setLance (false);
				this.listeGestionnaire.get (i).stopper (message);
			}
			else {
				this.listeGestionnaire.remove (i);
			}
		}
		for (int i = 0;i < this.listeCanaux.size ();i++) {
			if (!this.listeCanaux.get (i).estVide ()) {
				this.listeCanaux.get (i).vider ();
			}
			this.listeCanaux.remove (i);
		}

		this.nbGC = 0;
		this.nbCanaux = 0;
		this.listeCanaux = null;
		this.listeGestionnaire = null;
		this.lance = false;

		try {
			this.conn.close ();
		}
		catch (final SQLException e) {
			Serveur.logServeur.logException ("Serveur", "stopperServeur", e); //$NON-NLS-1$ //$NON-NLS-2$
		}
		finally {
			this.socket.close ();
			Serveur.logServeur.logMessage ("Arrêt du serveur"); //$NON-NLS-1$
			Serveur.logCanaux.fermer ();
			Serveur.logCoDeco.fermer ();
			Serveur.logServeur.fermer ();
		}
	}

	/**
	 * <p>
	 * Supprime un canal.
	 * </p>
	 * 
	 * @param nom
	 *            Nom du canal à supprimer.
	 *            
	 * @throws CanalInexistantException
	 *             Le canal n'existe pas.
	 */
	public void supprimerCanal (final String nom) throws CanalInexistantException {
		final int tmp = this.canalExistant (nom);

		if (tmp != -1) {
			try {
				this.listeCanaux.get (tmp).notifier (Evenement.SUPPRIME, null, null);
			}
			catch (final IOException e) {
				Serveur.logServeur.logMessage ("Erreur d'entrée sortie"); //$NON-NLS-1$
				Serveur.logServeur.logException ("Serveur", "supprimerCanal", e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			this.listeCanaux.get (tmp).vider ();
			this.listeCanaux.remove (tmp);
			this.nbCanaux--;
			Serveur.logCanaux.logMessage ("Le canal " + nom + " a été supprimé"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		else {
			throw new CanalInexistantException ();
		}
	}

	/**
	 * <p>
	 * Savoir si un utilisateur existe ou non.
	 * </p>
	 * 
	 * @param login
	 *            Login de l'utilisateur.
	 * @param mdp
	 *            Mot de passe de l'utilisateur.
	 * @param client
	 *            Utilisateur qui contiendra les informations trouvées.
	 *            
	 * @throws SQLException
	 *             Erreur de la BDD.
	 * @throws IdentifierException
	 *             La personne est déjà connecté.
	 * @throws NoSuchAlgorithmException
	 *             Algorithme du MD5 non trouvé
	 */
	public void verifierLoginMdp (final String login, final String mdp, final Utilisateur client) throws SQLException, IdentifierException, NoSuchAlgorithmException {
		boolean bool = false;

		for (int i = 0;i < this.listeGestionnaire.size ();i++) {
			if (login.equals (this.listeGestionnaire.get (i).getClient ().getLogin ())) {
				bool = true;
			}
		}
		if (!bool) {
			// Création d'un objet Statement
			final PreparedStatement state = this.conn.prepareStatement ("SELECT * FROM utilisateurs WHERE identifiant = ? AND active = '1'"); //$NON-NLS-1$
			state.setString (1, login);
			// L'objet ResultSet contient le résultat de la requête SQL
			final ResultSet result = state.executeQuery ();

			if (!result.next () || !result.getString ("mdp").equals (this.md5 (mdp))) { //$NON-NLS-1$
				result.close ();
				state.close ();
				throw new IdentifierException ();
			}
			Serveur.logCoDeco.logMessage ("Identification " + login + " reussie"); //$NON-NLS-1$ //$NON-NLS-2$
			client.setLogin (login);
			client.setAdministrateur (result.getInt ("droits") == 0); //$NON-NLS-1$
			result.close ();
			state.close ();
		}
		else {
			throw new IdentifierException ();
		}
	}
}
