/*
 *  PCUServeur for the PCU project.
 *  Copyright (C) 2010  PLU Julien
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  For questions: julien.plu@redaction-developpez.com
 */
package pcuserveur.lib;

/**
 * @author PLU Julien
 * @version 1.0 
 * 
 * <b>
 *     Représente un utilisateur. Un utilisateur est identifié par
 *     son login. Il peut être simple utilisateur ou administrateur.
 * </b>
 * <p>
 *     Un Utilisateur est caractérisé par les informations suivantes:
 *     <ul>
 *         <li>Un boolean désignant si l'utilisateur est un administrateur.</li>
 *         <li>Un String représentant le login de l'utilisateur.</li>
 *     </ul>
 * </p>
 * 
 * @see GestionnaireConnexion
 * @see Vide
 */
public class Utilisateur implements Vide {
	/**
	 * <p>
	 * Désigne si l'utilisateur est un administrateur ou non.
	 * </p>
	 * 
	 * @see Utilisateur#Utilisateur (String, boolean)
	 * @see Utilisateur#estAdministrateur ()
	 * @see Utilisateur#estVide ()
	 * @see Utilisateur#setAdministrateur (boolean)
	 * @see Utilisateur#vider ()
	 */
	boolean administrateur;
	/**
	 * <p>
	 * Login de l'utilisateur.
	 * </p>
	 * 
	 * @see Utilisateur#Utilisateur (String, boolean)
	 * @see Utilisateur#estVide ()
	 * @see Utilisateur#getLogin ()
	 * @see Utilisateur#setLogin (String)
	 * @see Utilisateur#vider ()
	 */
	private String login;

	/**
	 * <p>
	 * Constructeur par défaut.
	 * </p>
	 */
	public Utilisateur () {
		// Constructeur par defaut.
	}

	/**
	 * <p>
	 * Constructeur paramétré
	 * </p>
	 * 
	 * @param login1
	 *            Le login.
	 * @param administrateur1
	 *            Un boolean, <code>true</code> si l'utilisateur est
	 *            administrateur <code>false</code> sinon.
	 */
	public Utilisateur (final String login1, final boolean administrateur1) {
		this.login = login1;
		this.administrateur = administrateur1;
	}

	/**
	 * <p>
	 * Savoir si l'utilisateur est administrateur. Si l'utilisateur est vide, le
	 * résultat est indéterminé.
	 * </p>
	 * 
	 * @return <code>true</code> si l'utilisateur est administrateur,
	 *         <code>false</code> sinon
	 */
	public boolean estAdministrateur () {
		return this.administrateur;
	}

	/**
	 * <p>
	 * Savoir si l'utilisateur est vide.
	 * </p>
	 * 
	 * @return <code>true</code> si l'utilisateur est vide, <code>false</code>
	 *         sinon.
	 * @see Vide
	 */
	public boolean estVide () {
		return this.login == null;
	}

	/**
	 * <p>
	 * Obtenir le login, si il existe.
	 * </p>
	 * 
	 * @return Le login, ou <code>null</code> si l'utilisateur est vide.
	 */
	public String getLogin () {
		return this.login;
	}

	/**
	 * <p>
	 * Définir si l'utilisateur est, ou pas, un administrateur.
	 * </p>
	 * 
	 * @param administrateur1
	 *            Un booléen de valeur <code>true</code> si l'utilisateur est
	 *            administrateur <code>false</code> sinon.
	 */
	public void setAdministrateur (final boolean administrateur1) {
		this.administrateur = administrateur1;
	}

	/**
	 * <p>
	 * Définir le login.
	 * </p>
	 * 
	 * @param login1
	 *            Le login.
	 */
	public void setLogin (final String login1) {
		this.login = login1;
	}

	/**
	 * <p>
	 * Vider l'utilisateur. A utiliser avec beaucoup de précaution.
	 * </p>
	 */
	public void vider () {
		this.login = null;
		this.administrateur = false;
	}
}
